@@ -7,22 +7,19 @@
 #define NIO4R_H
 
 #include "ruby.h"
-#if HAVE_RUBY_IO_H
-# include "ruby/io.h"
-#else
-# include "rubyio.h"
-#endif
+#include "ruby/io.h"
 #include "libev.h"
 
 struct NIO_Selector
 {
-    struct ev_loop *ev_loop;
+    ev_loop *ev_loop;
     struct ev_timer timer; /* for timeouts */
     struct ev_io wakeup;
 
-    int wakeup_reader, wakeup_writer;
-    int closed, selecting;
     int ready_count;
+    int closed, selecting;
+    int wakeup_reader, wakeup_writer;
+    volatile int wakeup_fired;
 
     VALUE ready_array;
 };
@@ -41,16 +38,17 @@ struct NIO_Monitor
     struct NIO_Selector *selector;
 };
 
+struct NIO_ByteBuffer
+{
+    char *buffer;
+    int position, limit, capacity, mark;
+};
+
+
 #ifdef GetReadFile
 # define FPTR_TO_FD(fptr) (fileno(GetReadFile(fptr)))
 #else
-
-#if !HAVE_RB_IO_T || (RUBY_VERSION_MAJOR == 1 && RUBY_VERSION_MINOR == 8)
-# define FPTR_TO_FD(fptr) fileno(fptr->f)
-#else
 # define FPTR_TO_FD(fptr) fptr->fd
-#endif /* !HAVE_RB_IO_T */
-
 #endif /* GetReadFile */
 
 /* Thunk between libev callbacks in NIO::Monitors and NIO::Selectors */
